package cc.fascinated.controller;

import cc.fascinated.exception.impl.BadRequestException;
import cc.fascinated.platform.Platform;
import cc.fascinated.services.ScoreService;
import cc.fascinated.services.UserService;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@RestController
@RequestMapping(value = "/scores")
public class ScoresController {
    /**
     * The tracked score service to use.
     */
    @NonNull
    private final ScoreService scoreService;

    /**
     * The user service to use
     */
    @NonNull
    private final UserService userService;

    @Autowired
    public ScoresController(@NonNull ScoreService scoreService, @NonNull UserService userService) {
        this.scoreService = scoreService;
        this.userService = userService;
    }

    /**
     * A GET mapping to retrieve the top
     * scores for a platform
     *
     * @param platform the platform to get the scores from
     * @return the scores
     * @throws BadRequestException if there were no scores found
     */
    @ResponseBody
    @GetMapping(value = "/top/{platform}")
    public ResponseEntity<List<?>> getTopScores(@PathVariable String platform, @RequestParam(defaultValue = "false") boolean scoresonly) {
        return ResponseEntity.ok(scoreService.getTopRankedScores(Platform.Platforms.getPlatform(platform), 50, scoresonly));
    }

    /**
     * A GET mapping to retrieve the total
     * amount of scores for a platform
     *
     * @param platform the platform to get the scores from
     * @return the amount of scores
     * @throws BadRequestException if there were no scores found
     */
    @ResponseBody
    @GetMapping(value = "/count/{platform}")
    public ResponseEntity<?> getScoresCount(@PathVariable String platform) {
        return ResponseEntity.ok(scoreService.getTotalScores(Platform.Platforms.getPlatform(platform)));
    }

    /**
     * A GET mapping to retrieve the score
     * history for a leaderboard
     *
     * @param platform the platform to get the history from
     * @return the score history
     * @throws BadRequestException if there were no history found
     */
    @ResponseBody
    @GetMapping(value = "/history/{platform}/{playerId}/{leaderboardId}")
    public ResponseEntity<?> getScoreHistory(@PathVariable String platform, @PathVariable String playerId, @PathVariable String leaderboardId) {
        return ResponseEntity.ok(scoreService.getPreviousScore(
                Platform.Platforms.getPlatform(platform),
                userService.getUser(playerId),
                leaderboardId
        ).getRight());
    }

    /**
     * A GET mapping to retrieve the score
     * history for a leaderboard
     *
     * @param platform the platform to get the history from
     * @return the score history
     * @throws BadRequestException if there were no history found
     */
    @ResponseBody
    @GetMapping(value = "/history/{platform}/{playerId}")
    public ResponseEntity<?> getScoreHistory(@PathVariable String platform, @PathVariable String playerId) {
        return ResponseEntity.ok(scoreService.getImprovedScores(
                Platform.Platforms.getPlatform(platform),
                userService.getUser(playerId),
                30
        ));
    }

    /**
     * A GET mapping to retrieve the score
     * history for a leaderboard
     *
     * @param platform the platform to get the history from
     * @return the score history
     * @throws BadRequestException if there were no history found
     */
    @ResponseBody
    @GetMapping(value = "/improved/best/{platform}")
    public ResponseEntity<?> getBestImprovedScores(@PathVariable String platform) {
        return ResponseEntity.ok(scoreService.getBestImprovedScores(
                Platform.Platforms.getPlatform(platform),
                30
        ));
    }
}
